#include "Engine/PhongLighting.hlsl"

struct PixelOut {
  float4 A : SV_Target0;
  uint4 B  : SV_Target1;
};

PixelOut main(VertexIn In, float4 screenPos : SV_Position)
{
  SurfaceOut surface = SURFACE_FUNCTION(In);
#if defined(COGS_ALPHA_TEST_DISCARD) || defined(COGS_ALPHA_TO_COVERAGE)
  if (surface.Color.a < alphaThreshold) discard;
#endif

  if ((GetMaterialId() & COGS_LIGHT_MASK_PHONG) == COGS_LIGHT_MASK_PHONG) {

    float light = 0.5f;
    for (uint i = 0; i < COGS_MAX_LIGHTS; ++i) {
      float4 lightPosition = lightPositions[i];

      float shadow = 1.f;
#ifdef COGS_FORWARD_SHADOWS_ENABLED
      if ((GetMaterialId() & COGS_LIGHT_MASK_SHADOW_RECEIVER) == COGS_LIGHT_MASK_SHADOW_RECEIVER) {
        if(lightPositions[i].w == 0) {
#if COGS_NUM_DIRECTIONAL_SHADOWS > 0
          shadow = calculateShadowContribution(In.WorldPosition, surface.Normal, lightDirections[i].xyz, i, screenPos);
#endif
      }
      else {
#if COGS_NUM_POINT_SHADOWS > 0
          shadow = calculatePointLightShadow(lightPosition, In.WorldPosition.xyz, surface.Normal, i);
#endif
        }
      }
#endif
      float3 offset = In.WorldPosition.xyz - lightPosition.xyz;
      float3 lightDir = lightDirections[i].w == 0.f ? lightDirections[i].xyz : normalize(offset);

      light +=  0.5 * smoothstep(0.0, 0.05, shadow * saturate(dot(surface.Normal, -lightDir)));
    }
    surface.Color *= light;

    float3 n = 1024.0 * saturate(0.5 * ((float3)1 + mul(surface.Normal, (float3x3)viewMatrix)));

    PixelOut o = (PixelOut)0;
    o.A = surface.Color;
    o.B = uint4(n.x, n.y, n.z, 0);
    return o;
  }
  else {
    PixelOut o = (PixelOut)0;
    o.A = surface.Color;
    o.B = uint4(512, 512, 512, 0);
    return o;
  }
}
