#if defined(COGS_DEBUGVIEW_OBJECTID)
#include "Random.hlsl"
#endif

#if defined(COGS_DEBUGVIEW_SHADOW_CASCADES)
#include "Engine/PhongLighting.hlsl"
#endif


struct MyVertexIn
{
  float4 A : TEXCOORD8;
  float4 B : TEXCOORD9;
  float4 C : TEXCOORD10;
  VertexIn vertexIn;
};

struct PixelOut {
  float4 A : SV_Target0;
  uint4 B  : SV_Target1;
};

PixelOut main(MyVertexIn In, float4 ScreenPos : SV_Position)
{
  SurfaceOut surface = SURFACE_FUNCTION(In.vertexIn);

  float4 A = 0.5 * fwidth(In.A);
  float4 B = 0.5 * fwidth(In.B);
  float4 C = 0.5 * fwidth(In.C);

#if defined(COGS_ALPHA_TEST_DISCARD) || defined(COGS_ALPHA_TO_COVERAGE)
  if (surface.Color.a < alphaThreshold) discard;
#endif

  float na = dot((float4)(A > (float4)(0.001)), (float4)1.0);
  float nb = dot((float4)(B > (float4)(0.001)), (float4)1.0);
  float nc = dot((float4)(C > (float4)(0.001)), (float4)1.0);

  float e[12] = {
    In.A.x, In.A.y, In.A.z, In.A.w,
    In.B.x, In.B.y, In.B.z, In.B.w,
    In.C.x, In.C.y, In.C.z, In.C.w
  };

  float d[12] = {
    A.x, A.y, A.z, A.w,
    B.x, B.y, B.z, B.w,
    C.x, C.y, C.z, C.w,
  };

  float max_ = 0.0;
  for (int i = 0; i < 12; i++) {
    max_ = max(max_, d[i]);
  }

  float dist = 1.0;
  float min_ = max_;
  for (int j = 0; j < 12; j++) {
    if (0.001 < d[j]) {
      min_ = min(min_, d[j]);
      dist = min(dist, (1.0 / d[j]) * min(1.0 - e[j], e[j]));
    }
  }

  PixelOut o = (PixelOut)0; 
#if defined(COGS_DEBUGVIEW_TRIANGLESIZE)

  float4 D = (A + B + C);
  float a = 0.5 * (min_ + max_);
  float s = 1.0 / a;
  o.A = (float4)s;

#elif defined(COGS_DEBUGVIEW_OBJECTID)

  uint id = wangHash(objectId);
  o.A = float4((1.0 / 255.0) * float((id >> 0u) & 255u),
               (1.0 / 255.0) * float((id >> 8u) & 255u),
               (1.0 / 255.0) * float((id >> 16u) & 255u),
               1.0);

#elif defined(COGS_DEBUGVIEW_NORMAL)

  o.A = float4(0.5 * (surface.Normal + (float3)1), 1.0);

#elif defined(COGS_DEBUGVIEW_SHADOW_CASCADES)
  if ((GetMaterialId() & COGS_LIGHT_MASK_PHONG) == COGS_LIGHT_MASK_PHONG) {
    float3 color = float3(1, 1, 1);
    float shadowFactor = 1.0;

#ifdef COGS_FORWARD_SHADOWS_ENABLED
    for (uint i = 0; i < numLights.x; ++i) {

      ShadowData shadow = shadows[i];
      if ((shadow.enabled != 0) && (lightPositions[i].w == 0.0)) {
        float viewZ = dot(shadow.cascadeLine.xyz, In.vertexIn.WorldPosition.xyz) + shadow.cascadeLine.w;

        if (viewZ <= shadow.farSplits[shadow.numCascades - 1]) {
          int index = 0;
          for (int c = shadow.numCascades - 1; c >= 0; --c) {
            if (viewZ < shadow.farSplits[c]) {
              color = float3(((c + 1) & 1) != 0 ? 1.0 : 0.0,
                             ((c + 1) & 2) != 0 ? 1.0 : 0.0,
                             ((c + 1) & 4) != 0 ? 1.0 : 0.0);
              index = c;
            }
          }
        }

        float4 lightPosition = lightPositions[i];
        shadowFactor = calculateShadowContribution(In.vertexIn.WorldPosition, surface.Normal, lightDirections[i].xyz, i, o.A);
        break;
      }
    }
#endif

    o.A = float4(shadowFactor * color, 1.0);
  }
  else {
    o.A = float4(0.2, 0.2, 0.2, 1.0);
  }
#endif

  float3 n = 1024.0 * saturate(0.5 * ((float3)1 + mul(surface.Normal, (float3x3)viewMatrix)));
  if ((GetMaterialId() & COGS_LIGHT_MASK_PHONG) != COGS_LIGHT_MASK_PHONG) {
    n = (float3)512.0;
  }
  o.B = uint4(n.x, n.y, n.z, uint(max(0.0, 2.0 * dist)));

  return o;
}
