local Runtime = {}
Runtime.__index = Runtime

function Runtime:initializeTypes()
	local setters = {
		-- Values
		int32_t = bridge.setIntegerField,
		uint32_t = bridge.setIntegerField,
		float = bridge.setFloatField,
		double = bridge.setDoubleField,
		bool = bridge.setBoolField,
		-- Vectors
		vec2 = bridge.setVector2Field,
		vec3 = bridge.setVector3Field,
		vec4 = bridge.setVector4Field,
		quat = bridge.setQuaternionField,

		dvec2 = bridge.setVector2dField,
		dvec3 = bridge.setVector3dField,
		dvec4 = bridge.setVector4dField,

		-- Resources
		MeshHandle = bridge.SetMeshField,
		TextureHandle = bridge.SetTextureHandle,
		ModelHandle = bridge.SetModelField,
		MaterialInstanceHandle = bridge.SetMaterialInstanceField
	}

	self.settersById = {}

	for key, value in pairs(setters) do
		local typeId = self.bridge.getTypeId(key)
		if value then
			self.settersById[typeId] = value
		else
			console.log("Could not get type id for type " .. value);
		end
	end
end

function Runtime:initialize()
	self.bridge = bridge

	self:initializeTypes()

	self.entityHandler = EntityHandler.new(self)
	self.entityHandler:initializeEntities()

	self.resources = Resources.new(self)
end

function Runtime:destroyEntity(entity)
	self.bridge.destroyEntity(bridge.context, entity.id)
end

function Runtime.new()
	local r = {}
	setmetatable(r, Runtime)
	return r
end

runtime = Runtime.new()
