#include "AdaptivePlanarGrid.es30.glsl"

#if defined(COGS_BASIC_TERRAIN_ELEVATION_IN_RED)
float elevation(vec2 texpos)
{
  return textureLod(Elevation, texpos, 0.0).r;
}
#elif defined(COGS_BASIC_TERRAIN_ELEVATION_IN_ALPHA)
float elevation(vec2 texpos)
{
  return textureLod(Elevation, texpos, 0.0).a;
}
#elif defined(COGS_BASIC_TERRAIN_ELEVATION_IN_RED_GREEN_BLUE)
float elevation(vec2 texpos)
{
  vec3 value = textureLod(Elevation, texpos, 0.0).rgb;
  return 256.0 * 256.0 * 256.0 * value.r + 256.0 * 256.0 * value.g + 256.0 * value.b;
}
#endif

VertexOut vertexFunction(VertexIn In)
{
  // Snap edge vertices to neighbouring grids.
  vec2 pos = snapAdaptivePlanarGridEdges(In.Position.xyz).xy;

  // Get elevation.
  vec2 texCoord = getTexCoordUnit(pos);
  float e_raw = elevation(texCoord);

  float e = elevationTransform.x * e_raw + elevationTransform.y;

  vec4 surfacePositionWorld = worldMatrix * vec4(pos, e, 1.f);

  // Output results
  VertexOut vertex = createVertexOut();
  vertex.WorldPosition = surfacePositionWorld;
  vertex.Position = worldToClipMatrix * surfacePositionWorld;
  vertex.texCoord = vec3(texCoord, e_raw);

  return vertex;
}
