#include "Default/DefaultCommon.hlsl"

SurfaceOut surfaceFunction(in VertexIn In)
{
  SurfaceOut Out = (SurfaceOut)0;

  UpdateMaterialProperties(In, Out);

#ifdef COGS_DEFAULT_MATERIAL_TEXTURED
  float4 textureColor = diffuseMap.Sample(diffuseMapSampler, In.TexCoord * diffuseMapScale);

  Out.Color *= textureColor;
#endif

#ifdef DEFAULT_MATERIAL_VERTEX_COLOR
  Out.Color *= In.Color;
#endif

#ifdef DEFAULT_MATERIAL_INSTANCED_COLORS
  Out.Color *= In.InstanceColor;
#endif

#ifdef DEFAULT_MATERIAL_ENABLE_LIGHTING

#if defined(DEFAULT_MATERIAL_OBJECT_SPACE_NORMAL_MAP)
  Out.Normal = normalize(normalMap.Sample(normalMapSampler, In.TexCoord * normalMapScale).rgb);
#elif defined(BASE_MATERIAL_VERTEX_NORMAL0)
  float3 normal = normalize(In.Normal);
#ifdef DEFAULT_MATERIAL_TANGENT_SPACE_NORMAL_MAP
  float3 normalColor = 2.0 * normalMap.Sample(normalMapSampler, In.TexCoord * normalMapScale).rgb - 1.0;

  float3 tangent = normalize(In.Tangent);
  float3 biTangent = cross(normal, tangent);

  float3 bumpNormal = normalize(normalColor.z * normal + normalColor.x * tangent + normalColor.y * biTangent);
  normal = normalize(lerp(normal, bumpNormal, normalMapFactor));
#endif
  Out.Normal = In.Facing ? normal : -normal;
#endif

#endif

#ifdef DEFAULT_MATERIAL_SPECULAR_MAP
  float3 specularColor = specularMap.Sample(specularMapSampler, In.TexCoord * specularMapScale).rgb;

  Out.SpecularPower = 20 + max(specularColor.r, max(specularColor.g, specularColor.b)) * 80;
  Out.Specular = specularColor.rgb;
#endif

  return Out;
}
