#include "Bayer.es30.glsl"
#include "Random.es30.glsl"


float calculateShadow(in ShadowData shadow, in vec3 lightSpacePosition, in int index, in int o, in int Ns, in int Nn, in vec2 eta)
{
  if (Nn == 1) {
    if ((o == 0) && (Ns==1)) {
      return texture(cascadedShadowMap, vec4(lightSpacePosition.xy, index, lightSpacePosition.z));
    }
    return 0.0;
  }
  else {
    float rv = 0.0;

    if (Nn == 9) {
      const vec2 shifts[9] = vec2[9](
        vec2(  0.0,  0.0 ),
        vec2( -1.5,  0.0 ),
        vec2(  0.0, -1.5 ),
        vec2(  0.0,  1.5 ),
        vec2(  1.5,  0.0 ),
        vec2( -1.5, -1.5 ),
        vec2( -1.5,  1.5 ),
        vec2(  1.5, -1.5 ),
        vec2(  1.5,  1.5 )
      );

      for (int i = o; i < Ns; i++) {
        vec2 tp = lightSpacePosition.xy + mat2(eta.x, -eta.y, eta.y, eta.x) * vec2(shadow.texelSize) * shifts[i];
        rv += (1.0/9.0)*texture(cascadedShadowMap, vec4(tp, index, lightSpacePosition.z));
      }
    }
    else if (Nn == 36) {

      const vec2 shifts[36] = vec2[36](
        vec2( -2.5, -2.5 ), vec2( -2.5, -1.5 ), vec2( -2.5, -0.5 ), vec2( -2.5,  0.5 ), vec2( -2.5,  1.5 ), vec2( -2.5,  2.5 ),
        vec2( -1.5, -2.5 ), vec2( -1.5, -1.5 ), vec2( -1.5, -0.5 ), vec2( -1.5,  0.5 ), vec2( -1.5,  1.5 ), vec2( -1.5,  2.5 ),
        vec2( -0.5, -2.5 ), vec2( -0.5, -1.5 ), vec2( -0.5, -0.5 ), vec2( -0.5,  0.5 ), vec2( -0.5,  1.5 ), vec2( -0.5,  2.5 ),
        vec2(  0.5, -2.5 ), vec2(  0.5, -1.5 ), vec2(  0.5, -0.5 ), vec2(  0.5,  0.5 ), vec2(  0.5,  1.5 ), vec2(  0.5,  2.5 ),
        vec2(  1.5, -2.5 ), vec2(  1.5, -1.5 ), vec2(  1.5, -0.5 ), vec2(  1.5,  0.5 ), vec2(  1.5,  1.5 ), vec2(  1.5,  2.5 ),
        vec2(  2.5, -2.5 ), vec2(  2.5, -1.5 ), vec2(  2.5, -0.5 ), vec2(  2.5,  0.5 ), vec2(  2.5,  1.5 ), vec2(  2.5,  2.5 )
      );

      for (int i = o; i < Ns; i++) {
        vec2 tp = lightSpacePosition.xy + vec2(shadow.texelSize) * shifts[i];

        rv += (1.0 / 36.0)*texture(cascadedShadowMap, vec4(tp, index, lightSpacePosition.z));
      }
    }
    return rv;
  }
}


float calculateShadowContribution(in vec4 worldPosition, in vec3 normal, in vec3 lightDir, in int shadowIndex, in vec4 screenPos)
{
#if defined(COGS_DISABLE_DIRECTIONAL_SHADOWS)
  return 1;
#endif
  ShadowData shadow = shadows[shadowIndex];

  if (shadow.enabled == 0) return 1.0;

  float viewZ = dot(shadow.cascadeLine.xyz, worldPosition.xyz) + shadow.cascadeLine.w;

  if (viewZ > shadow.farSplits[shadow.numCascades - 1]) return 1.0;

  int index = 0;
  for (int c = shadow.numCascades - 1; c >= 0; --c) {
    if (viewZ < shadow.farSplits[c]) {
      index = c;
    }
  }

  int lightIndex = index + cascadeOffsets[shadowIndex].x;

  vec4 lightSpacePosition = shadow.lightMatrix[index] * vec4(worldPosition.xyz, 1);
  vec3 pos_l = (1.0 / lightSpacePosition.w)*lightSpacePosition.xyz;
  pos_l.z = clamp(pos_l.z - shadow.shadowSampleBias, 0.0, 1.0);

#if defined(COGS_SOFT_SHADOWS_HIGH)
  int Nn = 36;
#elif defined(COGS_SOFT_SHADOWS_LOW)
  int Nn = 9;
#else
  int Nn = 1;
#endif

  vec2 eta;
#if COGS_SOFT_SHADOWS_JITTER==1
  {
    float t_ = (6.28318530718 / 4294967295.0)*float(wangHash(1664525u * (uint(screenPos.y)) + uint(screenPos.x)));
    eta = vec2(cos(t_), sin(t_));
  }
#else
  eta = vec2(1, 0);
#endif

  float fade = viewZ - shadow.nearSplits[index];
  if (0.f < fade && index < shadow.numCascades - 1) {
    fade = (1.f / (shadow.farSplits[index] - shadow.nearSplits[index])) * fade;

    int Ns = min(Nn, int(float(Nn) * fade + (1.0 / 15.0)*float(bayer4(uvec2(screenPos.xy)))));

    float acc = calculateShadow(shadow, pos_l, lightIndex, Ns, Nn, Nn, eta);
    if (index < shadow.numCascades - 1) {
      vec4 lightSpacePosition = shadow.lightMatrix[index + 1] * vec4(worldPosition.xyz, 1);
      vec3 pos_l = (1.0 / lightSpacePosition.w) * lightSpacePosition.xyz;
      pos_l.z = clamp(pos_l.z - shadow.shadowSampleBias, 0.0, 1.0);
      acc += calculateShadow(shadow, pos_l, lightIndex + 1, 0, Ns, Nn, eta);
    }
    else {
      acc += float(Ns) / float(Nn);
    }
    return acc;
  }
  else {
    return calculateShadow(shadow, pos_l, lightIndex, 0, Nn, Nn, eta);
  }
}

float calculateShadowContribution(in float viewZ, in vec4 worldPositionA, in vec4 worldPositionB, in vec3 lightDir, in int shadowIndex, in vec4 screenPos)
{
#if defined(COGS_DISABLE_DIRECTIONAL_SHADOWS)
  return 1.0;
#endif
  ShadowData shadow = shadows[shadowIndex];

  if (shadow.enabled == 0) return 1.0;

  if (viewZ > shadow.farSplits[shadow.numCascades - 1]) return 1.0;

  int index = 0;
  for (int c = shadow.numCascades - 1; c >= 0; --c) {
    if (viewZ < shadow.farSplits[c]) {
      index = c;
    }
  }

  vec4 lightSpacePositionA = shadow.lightMatrix[index] * worldPositionA;
  lightSpacePositionA.xyz = (1.0 / lightSpacePositionA.w)*lightSpacePositionA.xyz;
  lightSpacePositionA.z -= shadow.shadowSampleBias;

  vec4 lightSpacePositionB = shadow.lightMatrix[index] * worldPositionB;
  lightSpacePositionB.xyz = (1.0 / lightSpacePositionB.w)*lightSpacePositionB.xyz;
  lightSpacePositionB.z -= shadow.shadowSampleBias;

  int N = int(clamp( (1.0/shadow.texelSize) * length(lightSpacePositionA.xy - lightSpacePositionB.xy), 1.0, 25.0));
  float accum = 0.0;
  float weight = 0.0;
  for (int i = 0; i < N; i++) {
    vec3 lightSpacePosition = mix(lightSpacePositionA.xyz, lightSpacePositionB.xyz, (float(i) + 0.5) / float(N));
    lightSpacePosition.z = clamp(lightSpacePosition.z, 0.0, 1.0);
    //accum += cascadedShadowMap.SampleCmpLevelZero(cascadedShadowSampler, vec3(lightSpacePosition.xy, index), lightSpacePosition.z);
    accum += texture(cascadedShadowMap, vec4(lightSpacePosition.xy, index, lightSpacePosition.z));
    weight += 1.0;
  }

  return accum / weight;
}
