#ifndef COGS_COMMON_GLSL
#define COGS_COMMON_GLSL

struct SurfaceOut
{
  vec4 Color;
  vec3 Specular;
  float SpecularPower;
  vec3 Emissive;
  vec3 Normal;

#ifdef COGS_USE_OBJECTID
  int objectId;
#endif

#ifdef COGS_CUSTOM_DEPTH_WRITE
  float Depth;
#endif
};

SurfaceOut createSurfaceOut()
{
  SurfaceOut t;
  t.Color = vec4(0, 0, 0, 1);
  t.Specular = vec3(0);
  t.SpecularPower = 0.0;
  t.Emissive = vec3(0);
  t.Normal = vec3(0);
#ifdef COGS_USE_OBJECTID
  t.objectId = int(objectId());
#endif

#ifdef COGS_CUSTOM_DEPTH_WRITE
  t.Depth = 0;
#endif
  return t;
}

//#define COGS_LIGHT_MASK_PHONG 1
//#define COGS_LIGHT_MASK_INDIRECT 64

#if BASE_MATERIAL_VERTEX_POSITION0 == COGS_FLOAT2
vec4 positionTypeAsFloat4(vec2 p) { return vec4(p, 0.0, 1.0); }
#elif BASE_MATERIAL_VERTEX_POSITION0 == COGS_FLOAT3
vec4 positionTypeAsFloat4(vec3 p) { return vec4(p, 1.0); }
#else
vec4 positionTypeAsFloat4(vec4 p) { return p; }
#endif

vec3 octDecode(vec2 f)
{
  // From https://twitter.com/Stubbesaurus/status/937994790553227264
  vec3 n = vec3(f.x, f.y, 1.0 - abs(f.x) - abs(f.y));
  float t = max(-n.z, 0.0);
  n.x += 0.0 <= n.x ? -t : t;
  n.y += 0.0 <= n.y ? -t : t;
  return normalize(n);
}

int GetMaterialId()
{
  int materialId = 0;

//#if defined(COGS_LIGHT_MODEL_PHONG)
//  materialId += COGS_LIGHT_MASK_PHONG;
//#elif defined(COGS_LIGHT_MODEL_STANDARD)
//  materialId += 4;
//#endif

#ifdef COGS_SURFACE_CLOSED
  materialId += 16;
#endif

//#ifdef COGS_INDIRECT_LIGHTING_FULL
//  materialId += COGS_LIGHT_MASK_INDIRECT;
//#endif

  return materialId;
}

#endif // COGS_COMMON_GLSL
