#include "Noise.hlsl"

Texture2D colorTexture;
Texture2D emissiveTexture;
Texture2D specularTexture;
Texture2D normalTexture;
Texture2D depthTexture;

float time;
float4x4 inverseProjectionMatrix;
float4x4 projectionMatrix;

SamplerState linearSampler;

struct VertexIn
{
  float4 Position : SV_Position;
  float4 ScreenPosition : TEXCOORD0;
  float2 TexCoords : TEXCOORD1;
};

float4 main(VertexIn In, float4 ScreenPos : SV_Position) : SV_Target
{
  const float4 color = colorTexture.Load(int3(ScreenPos.xy, 0));
  const float4 emissive = emissiveTexture.Load(int3(ScreenPos.xy, 0));
  const float4 specular = specularTexture.Load(int3(ScreenPos.xy, 0));
  const float4 normal = normalTexture.Load(int3(ScreenPos.xy, 0));
  const float depth = depthTexture.Load(int3(ScreenPos.xy, 0)).r;

  float w, h;
  depthTexture.GetDimensions(w, h);

  if (depth < 1.0 || true) {

    float4 viewSpacePosition00 = mul(float4(In.ScreenPosition.xy, depth, 1), inverseProjectionMatrix);
    viewSpacePosition00 /= viewSpacePosition00.w;

    float4 viewSpacePosition10 = mul(float4(In.ScreenPosition.xy + float2(8.0 / w, 0.0), depth, 1), inverseProjectionMatrix);
    viewSpacePosition10 /= viewSpacePosition10.w;

    float4 viewSpacePosition01 = mul(float4(In.ScreenPosition.xy + float2(0.0, 8.0 / h), depth, 1), inverseProjectionMatrix);
    viewSpacePosition01 /= viewSpacePosition01.w;

    float noise = pnoise(5 * viewSpacePosition00.xyz + float3(0, 10.0*frac(0.3*time), 0), (float3)10.0);

    float3 emitter = float3(1.0, 1.0, 0);

    float3 l00 = normalize(viewSpacePosition00.xyz - emitter);
    float3 l10 = normalize(viewSpacePosition10.xyz - emitter);
    float3 l01 = normalize(viewSpacePosition01.xyz - emitter);

    float jj00 = (64.0 / 3.14)*asin(l00.y);
    float jj01 = (64.0 / 3.14)*asin(l01.y);

    float ii00 = (256.0 / 3.14)*asin(l00.x) + frac(7 * time);
    float ii10 = (256.0 / 3.14)*asin(l10.x) + frac(7 * time);

    int j00 = (int)floor(jj00);
    int j01 = (int)floor(jj01);

    int i00 = (int)floor(ii00);
    int i10 = (int)floor(ii10);

    if ((j00 != j01) && (i00 != i10)) {

      float q = -0.2*viewSpacePosition00.z + 0.4*noise;

      if (q < 4.0 && 0.1 < abs(noise)) {
        return float4(max(0, 1 - q), 1 - abs(q - 1.5), 1 - abs(q - 3.0), 1);
      }
    }

    /*

    float d = 1-viewSpacePosition.z/10.0;
    float t = asin(l.y);
    float s = frac(20.0*asin(q.x));



    for (int i = 0; i < 16; i++) {
      float g = (3.14 / 4.0)*((i - 7.5) / 15.0);
      if (abs(t - g) < 0.001  && s < 0.7) {
        return float4(0.5*d, 1.2*d, 0.2*d, 1.0);
      }
    }*/
  }
  return 0.1*float4(color);
}
